@@ -161,8 +161,16 @@ static int ui_alloc(struct ui_st **stp)
 }
 
 
+static int output_handler(const char *str)
+{
+	return print_handler(str, str_len(str), NULL);
+}
+
+
 static struct ui ui_wincons = {
-	.name = "wincons"
+	LE_INIT,
+	"wincons",
+	output_handler
 };
 
 
