@@ -799,6 +799,8 @@ void ua_hangup(struct ua *ua, struct call *call,
 
 	(void)call_hangup(call, scode, reason);
 
+	ua_event(ua, UA_EVENT_CALL_CLOSED, call, reason);
+
 	mem_deref(call);
 
 	resume_call(ua);
@@ -909,7 +911,9 @@ int ua_options_send(struct ua *ua, const char *uri,
 	if (!dialbuf)
 		return ENOMEM;
 
-	err |= uri_complete(ua, dialbuf, uri);
+	err = uri_complete(ua, dialbuf, uri);
+	if (err)
+		goto out;
 
 	dialbuf->buf[dialbuf->end] = '\0';
 
@@ -921,6 +925,7 @@ int ua_options_send(struct ua *ua, const char *uri,
 		warning("ua: send options: (%m)\n", err);
 	}
 
+ out:
 	mem_deref(dialbuf);
 
 	return err;
