@@ -19,17 +19,38 @@ struct test {
 #define TEST(a) {a, #a}
 
 static const struct test tests[] = {
-	TEST(test_cmd),
-	TEST(test_ua_alloc),
-	TEST(test_uag_find_param),
-	TEST(test_ua_register),
-	TEST(test_cplusplus),
-	TEST(test_call_answer),
-	TEST(test_call_reject),
 	TEST(test_call_af_mismatch),
+	TEST(test_call_answer),
+	TEST(test_call_answer_hangup_a),
+	TEST(test_call_answer_hangup_b),
+	TEST(test_call_reject),
+	TEST(test_cmd),
+	TEST(test_cplusplus),
+	TEST(test_ua_alloc),
+	TEST(test_ua_register),
+	TEST(test_uag_find_param),
 };
 
 
+static int run_one_test(const struct test *test)
+{
+	int err;
+
+	re_printf("[ RUN      ] %s\n", test->name);
+
+	err = test->exec();
+	if (err) {
+		warning("%s: test failed (%m)\n",
+			test->name, err);
+		return err;
+	}
+
+	re_printf("[       OK ]\n");
+
+	return 0;
+}
+
+
 static int run_tests(void)
 {
 	size_t i;
@@ -53,11 +74,45 @@ static int run_tests(void)
 }
 
 
+static void test_listcases(void)
+{
+	size_t i, n;
+
+	n = ARRAY_SIZE(tests);
+
+	(void)re_printf("\n%zu test cases:\n", n);
+
+	for (i=0; i<(n+1)/2; i++) {
+
+		(void)re_printf("    %-32s    %s\n",
+				tests[i].name,
+				(i+(n+1)/2) < n ? tests[i+(n+1)/2].name : "");
+	}
+
+	(void)re_printf("\n");
+}
+
+
+static const struct test *find_test(const char *name)
+{
+	size_t i;
+
+	for (i=0; i<ARRAY_SIZE(tests); i++) {
+
+		if (0 == str_casecmp(name, tests[i].name))
+			return &tests[i];
+	}
+
+	return NULL;
+}
+
+
 static void usage(void)
 {
 	(void)re_fprintf(stderr,
-			 "Usage: selftest [options]\n"
+			 "Usage: selftest [options] <testcases..>\n"
 			 "options:\n"
+			 "\t-l               List all testcases and exit\n"
 			 "\t-v               Verbose output (INFO level)\n"
 			 );
 }
@@ -66,6 +121,7 @@ static void usage(void)
 int main(int argc, char *argv[])
 {
 	struct config *config;
+	size_t i, ntests;
 	int err;
 
 	err = libre_init();
@@ -75,7 +131,7 @@ int main(int argc, char *argv[])
 	log_enable_info(false);
 
 	for (;;) {
-		const int c = getopt(argc, argv, "v");
+		const int c = getopt(argc, argv, "hlv");
 		if (0 > c)
 			break;
 
@@ -86,6 +142,10 @@ int main(int argc, char *argv[])
 			usage();
 			return -2;
 
+		case 'l':
+			test_listcases();
+			return 0;
+
 		case 'v':
 			log_enable_info(true);
 			break;
@@ -95,8 +155,13 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	if (argc >= (optind + 1))
+		ntests = argc - optind;
+	else
+		ntests = ARRAY_SIZE(tests);
+
 	re_printf("running baresip selftest version %s with %zu tests\n",
-		  BARESIP_VERSION, ARRAY_SIZE(tests));
+		  BARESIP_VERSION, ntests);
 
 	/* note: run SIP-traffic on localhost */
 	config = conf_config();
@@ -111,16 +176,39 @@ int main(int argc, char *argv[])
 	if (err)
 		goto out;
 
-	err = run_tests();
-	if (err)
-		goto out;
+	if (argc >= (optind + 1)) {
+
+		for (i=0; i<ntests; i++) {
+			const char *name = argv[optind + i];
+			const struct test *test;
+
+			test = find_test(name);
+			if (test) {
+				err = run_one_test(test);
+				if (err)
+					goto out;
+			}
+			else {
+				re_fprintf(stderr,
+					   "testcase not found: `%s'\n",
+					   name);
+				err = ENOENT;
+				goto out;
+			}
+		}
+	}
+	else {
+		err = run_tests();
+		if (err)
+			goto out;
+	}
 
 #if 1
 	ua_stop_all(true);
 #endif
 
 	re_printf("\x1b[32mOK. %zu tests passed successfully\x1b[;m\n",
-		  ARRAY_SIZE(tests));
+		  ntests);
 
  out:
 	if (err) {
